#
#     Sam Hadow - Huffman-py
#     Copyright (C) 2023
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import unittest
from huffman_py.Node import *
from huffman_py.Tree import *
from huffman_py.functions.encode import *
from huffman_py.functions.decode import *
from huffman_py.functions.occurence import *

# pour lancer les tests utilisez:
# python -m unittest discover

class TestUtils(unittest.TestCase):
    def test_Tree_id(self):
        a = Node(10, 'a')
        b = Node(8,'b')
        r1 = Node(18,'',left=b,right=a)

        tree1 = Tree(r1)

        # vérification affectation d'un identifier unique en créant l'tree
        liste_id = []
        verification_id = True
        for elem in tree1.nodes:
            liste_id.append(elem.identifier)
        if len(liste_id) > len(set(liste_id)):
            verification_id  = False

        self.assertTrue(verification_id)

    def test_Tree_fusion(self):
        # fusion des trees
        # on doit retrouver les éléments des 2 trees + une nouvelle root
        # les identifiers doivent toujours être uniques
        a = Node(10, 'a')
        b = Node(8,'b')
        r1 = Node(18,'',left=b,right=a)
        c = Node(15, 'c')
        d = Node(20,'d')
        r2 = Node(18,'',left=d,right=c)

        tree1 = Tree(r1)
        tree2 = Tree(r2)

        l1 = len(tree1.nodes)
        l2 = len(tree2.nodes)
        tree1 += tree2
        verification_fusion = True
        if l1+l2+1 != len(tree1.nodes):
            verification_fusion = False

        liste_id2 = []
        verification_id2 = True
        for elem in tree1.nodes:
            liste_id2.append(elem.identifier)
        if len(liste_id2) > len(set(liste_id2)):
            verification_id2  = False

        self.assertTrue(verification_fusion)
        self.assertTrue(verification_id2)

    def test_Tree_seek(self):
        # seek de node
        a = Node(10, 'a')
        b = Node(8,'b')
        r1 = Node(18,'',left=b,right=a)

        tree1 = Tree(r1)

        self.assertEqual(tree1.seek(a),a)
        self.assertNotEqual(tree1.seek(b),a)

    def test_Tree_suppression(self):
        a = Node(10, 'a')
        b = Node(8,'b')
        r1 = Node(18,'',left=b,right=a)
        r2 = Node(18,'',left=None,right=r1)

        tree1 = Tree(r2)

        tree1 -= r1
        self.assertEqual(tree1.seek(a),None)
        self.assertEqual(tree1.seek(b),None)
        self.assertEqual(tree1.seek(r1),None)
        self.assertEqual(tree1.seek(r2),r2)

    def test_occurences(self):
        o1 = calcul_occurence('aaabcc')
        o2 = calcul_occurence('bacaac')
        # dans les 2 cas on doit avoir le même dictionnaire
        # 3 pour a, 2 pour c, 1 pour b
        self.assertEqual(o1,o2)
        self.assertEqual(o1['c'],2)
        self.assertEqual(o1['b'],1)
        self.assertEqual(o1['a'],3)


    def test_encodage_huffman(self):
        string = 'mouton'
        string2 = 'vache'
        (encodedOutput, root, huffmanEncoding) = huffman_encode(string)
        (encodedOutput2, root2, huffmanEncoding2) = huffman_encode(string2)
        # l'encodage doit être différent (les dictionnaires et trees aussi)
        self.assertNotEqual(huffmanEncoding, huffmanEncoding2)
        self.assertNotEqual(encodedOutput, encodedOutput2)
        self.assertNotEqual(Tree(root),Tree(root2))

    def test_decodage_huffman(self):
        string = 'chèvre'
        (encodedOutput, root, huffmanEncoding) = huffman_encode(string)
        # on doit être capable de décoder avec la root de l'tree ou avec le dictionnaire
        self.assertEqual(string,huffman_decode(encodedOutput,root))
        self.assertEqual(string,decode_from_dict(encodedOutput,huffmanEncoding))

        # on doit être capable de détecter si le dictionnaire/tree n'est  pas celui correspondant à un texte encodé
        string2 = 'poule'
        (encodedOutput2, root2, huffmanEncoding2) = huffman_encode(string2)
        with self.assertRaises(ValueError):
            decode_from_dict(encodedOutput2,huffmanEncoding)
        with self.assertRaises(ValueError):
            huffman_decode(encodedOutput2,root)


if __name__ == '__main__':
    unittest.main()
